from java.util.logging import Level

from org.darenet.thexbot import TheXBot
from org.darenet.thexbot.exceptions import ModuleException
from org.darenet.thexbot.irc.events import Command
from org.darenet.thexbot.scripting import Module

class Control(Module):
    """This module defines the control pseudo-client."""

    class DieCommand(Command):
        def __init__(self, client):
            Command.__init__(self, client, 'DIE', 0, 'Terminates X')

        def run(self, source, args):
            self.client.notice(source, 'Selfdestruct in 3... 2... 1...')
            TheXBot.shutdown()


    class ModloadCommand(Command):
        def __init__(self, client):
            Command.__init__(self, client, 'MODLOAD', 1, 'Loads a module')

        def run(self, source, args):
            self.client.notice(source,
                    "Attempting to load module %s..." % args[0])

            try:
                TheXBot.mm().loadModule(args[0])
            except Exception, e:
                self.client.notice(source,
                        "Could not load module %s: %s" % (args[0],
                            e))
                return
            except ModuleException, e:
                self.client.notice(source,
                        "Could not load module %s: %s" % (args[0],
                            e))
                return

            self.client.notice(source,
                    "Module %s loaded successfully." % args[0])


    class ModunloadCommand(Command):
        def __init__(self, client):
            Command.__init__(self, client, 'MODUNLOAD', 1, 'Unloads a module')

        def run(self, source, args):
            self.client.notice(source,
                    "Attempting to unload module %s..." % args[0])

            if TheXBot.mm().unloadModule(args[0]):
                self.client.notice(source,
                        "Module %s unloaded successfully." % args[0])
            else:
                self.client.notice(source,
                        "Module %s was not loaded." % args[0])


    class ModreloadCommand(Command):
        def __init__(self, client):
            Command.__init__(self, client, 'MODRELOAD', 1, 'Reloads a module')

        def run(self, source, args):
            TheXBot.mm().unloadModule(args[0])

            try:
                TheXBot.mm().loadModule(args[0])
            except Exception, e:
                self.client.notice(source,
                        "Could not load module %s: %s" % (args[0],
                            e))
                return

            self.client.notice(source,
                    "Module %s (re)loaded successfully." % args[0])


    def __init__(self, logger, proto):
        self.protocol = proto
        self.logger = logger
        self.client = self.createPseudoClient(proto, 'M', 'control',
                'control.darenet', 'X module control bot')
        self.commands = (self.DieCommand(self.client),
                self.ModloadCommand(self.client),
                self.ModreloadCommand(self.client),
                self.ModunloadCommand(self.client),
                )

        for command in self.commands:
            self.client.addCommand(command)

        self.logger.log(Level.WARNING, 'Initialized Control module.')

    def __deinit__(self):
        self.removePseudoClient(self.client)

__module__ = Control(jlogger, protocol)

