package invalid.thexbot;

import java.util.Properties;
import invalid.thexbot.exceptions.ConfigurationException;

/**
 * Handles dealing with configuration file directives.
 */
public final class ConfigurationManager {
	private final Properties props;
	private static final String[] required_properties = {
		"me.name", "me.numeric", "me.desc", "uplink.password", "x.debug", "x.modules"
	};

	/**
	 * Initializes the configuration manager.
	 *
	 * @param props the properties file to use for the configuration
	 * @throws ConfigurationException if a required directive is missing
	 */
	public ConfigurationManager(final Properties props) throws ConfigurationException
	{
		this.props = props;

		for (final String directive : required_properties) {
			if (this.props.getProperty(directive) == null) {
				throw new RuntimeException("Missing required configuration property: " + directive);
			}
		}
	}

	public String getServerName()
	{
		return this.props.getProperty("me.name");
	}

	public String getServerDescription()
	{
		return this.props.getProperty("me.desc");
	}

	public String getSID()
	{
		return this.props.getProperty("me.numeric");
	}

	public String getUplinkPassword()
	{
		return this.props.getProperty("uplink.password");
	}

	public boolean isDebug()
	{
		return this.props.getProperty("x.debug").equals("true");
	}
	
	public String getUserCloak()
	{
		return this.props.getProperty("network.user_cloak");
	}
	
	public String getOperCloak()
	{
		return this.props.getProperty("network.oper_cloak");
	}
	
	public String[] getModuleNames()
	{
		return this.props.getProperty("x.modules").split(",");
	}

	/**
	 * Returns the directive with the given name. Use this function only if there is no other
	 * function for what you seek with a descriptive name.
	 *
	 * @param directive the name of the directive to search
	 * @return the value as String or null if not found
	 */
	public String getDirective(final String directive)
	{
		return this.props.getProperty(directive);
	}
}
