package invalid.thexbot;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;

/**
 * Wrapper around JUL.
 *
 * @author Adam
 * @since 0.5.0
 */
class LoggerHandler extends Handler {
	private static final DateFormat fmt = new SimpleDateFormat("E M kk:mm:ss:SSSS yyyy");

	public LoggerHandler()
	{
		this.setLevel(Level.ALL);
	}

	@Override
	public void publish(LogRecord record)
	{
		if (TheXBot.conf != null && !TheXBot.conf.isDebug()
			&& record.getLevel().intValue() < Level.INFO.intValue()) {
			return;
		}

		StackTraceElement[] trace =
			(record.getThrown() != null ? record.getThrown().getStackTrace() : null);

		System.out.println(fmt.format(new Date(record.getMillis())) + " ("
			+ (record.getLoggerName() == null ? "(null)" : record.getLoggerName()) + ":"
			+ record.getLevel().getName() + "): "
			+ (record.getMessage() == null ? "" : record.getMessage()));
		if (trace != null) {
			System.out.println(record.getThrown().toString());

			for (final StackTraceElement e : trace) {
				System.out.println(e.toString());
			}
		}
	}

	@Override
	public void flush()
	{
	}

	@Override
	public void close() throws SecurityException
	{
	}
}

public class Logger extends java.util.logging.Logger {
	private static final Map<String, Logger> loggers = new HashMap<>();
	private static final Handler handler = new LoggerHandler();

	protected Logger(final String name, final String resourceBundle)
	{
		super(name, resourceBundle);
	}

	public static Logger getLogger(final String name)
	{
		Logger l = loggers.get(name);
		if (l != null) {
			return l;
		}

		l = new Logger(name, null);
		loggers.put(name, l);
		l.setLevel(Level.ALL);
		l.addHandler(handler);

		return l;
	}
}
