package invalid.thexbot.irc;

/**
 * Keeps track of various account metadata.
 *
 * @since 0.5.0
 */
public final class AccountInfo {
	private String name;
	private final int id; // -1 if id is not used by ircd
	private int created; // -1 if not used

	/**
	 * Creates new account info.
	 *
	 * @param name the name of the account (not null)
	 * @param id the unique id of the account; if IRCd does not support this, set -1
	 * @param created the creation timestamp of the account; if the IRCd does not support this,
	 * set -1
	 */
	public AccountInfo(String name, int id, int created)
	{
		if (name == null) {
			throw new IllegalArgumentException("name may not be null");
		}

		this.name = name;
		this.id = id;
		this.created = created;
	}

	/**
	 * @return the name
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name)
	{
		this.name = name;
	}

	/**
	 * @return the id
	 */
	public int getID()
	{
		return id;
	}

	/**
	 * @return the created
	 */
	public int getCreated()
	{
		return created;
	}

	/**
	 * @param created the created to set
	 */
	public void setCreated(int created)
	{
		this.created = created;
	}
}
