package invalid.thexbot.irc;

import java.util.HashSet;
import java.util.Set;

/**
 * Describes an IRC server.
 *
 * @since 0.5.0
 */
public final class Server implements Source {
	private final String name, desc, numeric;
	private final Server uplink;
	private boolean is_bursting;
	private final Set<User> users = new HashSet<>();
	private final Set<Server> downlinks = new HashSet<>();

	/**
	 * Creates a new server.
	 *
	 * @param name the server name
	 * @param desc the server description
	 * @param numeric the server numeric as String
	 * @param uplink the Server this server is linked to
	 */
	public Server(final String name, final String desc, final String numeric, final Server uplink)
	{
		this.name = name;
		this.desc = desc;
		this.numeric = numeric;
		this.uplink = uplink;
		
		this.is_bursting = true;
	}

	public String getName()
	{
		return this.name;
	}

	public String getDescription()
	{
		return this.desc;
	}

	public String getNumeric()
	{
		return this.numeric;
	}

	public Server getUplink()
	{
		return this.uplink;
	}
	
	public int getUserCount()
	{
		return this.users.size();
	}
	
	public Set<User> getUsers()
	{
		return this.users;
	}

	public void addUser(User u)
	{
		this.users.add(u);
	}
	
	public void removeUser(User u)
	{
		this.users.remove(u);
	}
	
	public boolean isBursting()
	{
		return this.is_bursting;
	}
	
	public void finishBurst()
	{
		this.is_bursting = false;
	}
	
	/**
	 * @return the Servers directly linked to this one
	 */
	public Set<Server> getDownlinks()
	{
		return this.downlinks;
	}
	
	/**
	 * Adds a downlink to the list.
	 * @param s the downlink to add
	 */
	public void addDownlink(Server s)
	{
		this.downlinks.add(s);
	}
	
	@Override
	public boolean isServer()
	{
		return true;
	}
}
