package invalid.thexbot.irc;

/**
 * Defines the abstract user modes a user can have.
 *
 * @since 0.5.0
 */
public enum UserMode {
	/**
	 * User is invisible from /who queries unless in common channels.
	 *
	 * @since 0.5.0
	 */
	INVISIBLE,
	/**
	 * User is an IRC operator.
	 *
	 * @since 0.5.0
	 */
	OPER,
	/**
	 * User is a server administrator.
	 *
	 * @since 0.5.0
	 */
	ADMIN,
	/**
	 * User is a network administrator.
	 *
	 * @since 0.5.0
	 */
	NETADMIN,
	/**
	 * User is a service.
	 *
	 * @since 0.5.0
	 */
	SERVICE,
	/**
	 * User wants to get a hidden host. (+x on P10)
	 *
	 * <p>Do not use this for UnrealIRCd-style cloaking, which is always transmitted as the
	 * shown host anyway, so we don't need to know about it.</p>
	 * 
	 * <p>Also note that this mode may be set, but that does not imply that the user's host has
	 * actually been changed: It is possible that users set +x without being authed, in which
	 * case they have the mode but no host change.</p>
	 *
	 * @since 0.5.0
	 */
	HIDEHOST,
	/**
	 * User receives no channel messages.
	 *
	 * @since 0.5.0
	 */
	DEAF,
	/**
	 * User has a fake host
	 *
	 * @since 0.5.0
	 */
	VHOST;
}
