package invalid.thexbot.irc.events;

import invalid.thexbot.irc.PseudoClient;
import invalid.thexbot.irc.User;

/**
 * Defines a command, invoked in private message or in a channel.
 *
 * @since 0.5.0
 */
public abstract class Command {
	private final String name;
	private final int min_args;
	private final String shorthelp;
	/**
	 * The {@link PseudoClient} we belong to. Visibility because of Python scripting.
	 */
	public final PseudoClient client;

	/**
	 * Creates a command.
	 *
	 * @param client the
	 * @{link PseudoClient} we belong to
	 * @param name the name of the command
	 * @param min_args the minimum number of arguments required to run the command
	 * @param shorthelp a short (&lt;80 chars) command help
	 */
	public Command(PseudoClient client, String name, int min_args, String shorthelp)
	{
		this.client = client;
		this.name = name;
		this.min_args = min_args;
		this.shorthelp = shorthelp;
	}

	/**
	 * Sends a long help text to the given {@link User}.
	 *
	 * @param u the User to talk to
	 */
	public void fullHelp(User u)
	{
		this.client.notice(u, "No help available for \002" + this.getName() + "\002.");
	}

	/**
	 * Runs this command.
	 *
	 * @param source the {@link User} who executed the command
	 * @param args the arguments to the command
	 */
	public abstract void run(User source, String[] args);

	/**
	 * @return the name
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * @return the minimum number of arguments
	 */
	public int getMinArgs()
	{
		return min_args;
	}

	/**
	 * @return the short help
	 */
	public String getShortHelp()
	{
		return shorthelp;
	}
}
