package invalid.thexbot.protocol;

import invalid.thexbot.protocol.darenetprotocol.MHandler;
import invalid.thexbot.protocol.darenetprotocol.BHandler;
import invalid.thexbot.protocol.darenetprotocol.SERVERHandler;
import invalid.thexbot.protocol.darenetprotocol.ERRORHandler;
import invalid.thexbot.protocol.darenetprotocol.EBHandler;
import invalid.thexbot.protocol.darenetprotocol.KHandler;
import invalid.thexbot.protocol.darenetprotocol.SNHandler;
import invalid.thexbot.protocol.darenetprotocol.SHandler;
import invalid.thexbot.protocol.darenetprotocol.MKHandler;
import invalid.thexbot.protocol.darenetprotocol.GHandler;
import invalid.thexbot.protocol.darenetprotocol.JHandler;
import invalid.thexbot.protocol.darenetprotocol.SQHandler;
import invalid.thexbot.protocol.darenetprotocol.CHandler;
import invalid.thexbot.protocol.darenetprotocol.QHandler;
import invalid.thexbot.protocol.darenetprotocol.WHandler;
import invalid.thexbot.protocol.darenetprotocol.LHandler;
import invalid.thexbot.protocol.darenetprotocol.NHandler;
import invalid.thexbot.protocol.darenetprotocol.ACHandler;
import invalid.thexbot.protocol.darenetprotocol.SJHandler;
import invalid.thexbot.protocol.darenetprotocol.PHandler;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import invalid.thexbot.Logger;
import invalid.thexbot.TheXBot;
import invalid.thexbot.Util;
import invalid.thexbot.irc.Server;
import invalid.thexbot.exceptions.ConfigurationException;
import invalid.thexbot.exceptions.ParserException;
import invalid.thexbot.exceptions.ResourceException;
import invalid.thexbot.irc.AccountInfo;
import invalid.thexbot.irc.Channel;
import invalid.thexbot.irc.PseudoClient;
import invalid.thexbot.irc.User;
import invalid.thexbot.irc.UserMode;
import invalid.thexbot.protocol.MessageHandler.SourceType;
import static invalid.thexbot.protocol.MessageHandler.SourceType.USER;

public class DarenetProtocol extends ProtocolBase {
	private final Server me;
	private final Map<String, MessageHandler> message_handlers;
	private static final Logger log = Logger.getLogger(DarenetProtocol.class.getName());
	private static int client_count = 1;

	@Override
	public void handshake()
	{
		sendLine("PASS :" + TheXBot.conf.getUplinkPassword());
		sendFormattedLine("SERVER %s 1 %d %d J10 %s]]] +ns6 :%s", me.getName(),
			TheXBot.start_time, Util.getTimestamp(), me.getNumeric(), me.getDescription());
		TheXBot.em().addServer(this.me);
		TheXBot.bursting = true;
	}
	
	@Override
	public void disconnect()
	{
		sendFormattedLine("%s SQ %s 0 :Shutting down", me.getNumeric(), me.getNumeric());
		super.disconnect();
	}

	@Override
	public void burst()
	{
		for (final PseudoClient u : TheXBot.em().getPseudoClients()) {
			introducePseudoClient(u);
		}
		endOfBurst();
	}

	@Override
	public String generateNumeric() throws ResourceException
	{
		if (Util.isMaxClientNumber(client_count)) {
			client_count = 1;
		}

		int tries = 1;
		String numeric = this.me.getNumeric() + Util.inttobase64(client_count++, 3);
		while (TheXBot.em().findUser(numeric) != null) {
			numeric = this.me.getNumeric() + Util.inttobase64(client_count++, 3);

			if (Util.isMaxClientNumber(tries)) {
				throw new ResourceException("Space for user numerics exhausted?!");
			}
			continue;
		}

		return numeric;
	}

	@Override
	public void acknowledgeEndOfBurst()
	{
		sendFormattedLine("%s EA", me.getNumeric());
		TheXBot.bursting = false;
	}

	@Override
	public void endOfBurst()
	{
		sendFormattedLine("%s EB", me.getNumeric());
		me.finishBurst();
		TheXBot.setState(TheXBot.XState.RUNNING);
	}

	private void pong(final String s)
	{
		sendFormattedLine("%s Z %s %s", me.getNumeric(), me.getNumeric(), s);
	}

	@Override
	public void pong(final User u)
	{
		pong(u.getNick());
	}

	@Override
	public void pong(final Server s)
	{
		pong(s.getName());
	}

	@Override
	public void numeric(final int num, final String message)
	{
		sendFormattedLine("%s %d %s", me.getNumeric(), num, message);
	}

	@Override
	public void introducePseudoClient(final PseudoClient u)
	{
		sendFormattedLine("%s N %s 1 %d %s %s +%s ]]]]]] %s :%s",
			me.getNumeric(), u.getNick(), Util.getTimestamp(), u.getRealIdent(),
			u.getRealHost(), getUserModeString(u), u.getNumeric(), u.getGECOS());
	}
	
	@Override
	public void quit(final PseudoClient u)
	{
		sendFormattedLine("%s Q :Connection closed", u.getNumeric());
	}

	private void sendMessage(final PseudoClient source, final User u, final String command,
		final String message)
	{
		for (String part : message.split("\n")) {
			/* The maximum message length is 512, including trailing \r\n, leaving only
			 * a real 510 characters (bytes). We'll have to work with byte[] here
			 * because in Java, sizeof char is possibly greater than sizeof byte.
			 * 
			 * However, we can't calculate the amount of bytes left for a message with
			 * just numerics; what matters is how much space is left before cutoff for
			 * the receiving user, so we must use the client protocol for our
			 * calculations since the P10 variant is guaranteed to be shorter.
			 * 
			 * Nick etc. are in ASCII, so from the 510 bytes, we must give up:
			 * 1 (:), source nick length, 1 (!), source shown ident length, 1 (@),
			 * source shown host length, 9 (" NOTICE :")
			 */
			int available_length = 400; // guesstimate for unsupported encoding
			try {
				available_length = 510 - 12 - (source.getNick().getBytes("ASCII").length
					+ source.getShownIdent().getBytes("ASCII").length
					+ source.getShownHost().getBytes("ASCII").length);
			} catch (UnsupportedEncodingException e) {
				log.log(Level.SEVERE, "getBytes for ASCII encoding impossible,"
					+ " breaking notice length calculations");
			}

			/* Not specifying encoding here because the box's native encoding will
			 * be the part that matters. Maybe, not sure in what encoding Java will send
			 * strings over the 'net.
			 */
			if (part.getBytes().length < available_length) {
				sendFormattedLine("%s %s %s :%s", source.getNumeric(), command,
					u.getNumeric(), part);
				continue;
			}

			/* It's impossible to cut the String up by byte length without violating
			 * any encoding's character length. We'll just pray whatever gets cut off
			 * isn't too horrible important.
			 */
			while (!part.isEmpty()) {
				if (part.length() < available_length) {
					sendFormattedLine("%s %s %s :%s", source.getNumeric(), command,
						u.getNumeric(), part);
					break;
				}

				String sub = part.substring(0, available_length - 1);
				sendFormattedLine("%s %s %s :%s", source.getNumeric(), command,
					u.getNumeric(), part);
				part = sub;
			}
		}
	}

	@Override
	public void sendPrivmsg(final PseudoClient source, final User u, final String message)
	{
		sendMessage(source, u, "P", message);
	}

	@Override
	public void sendNotice(final PseudoClient source, final User u, final String message)
	{
		sendMessage(source, u, "O", message);
	}

	/**
	 * Initializes the protocol. Note that you must manually call the handshake() function.
	 *
	 * @param sock the socket to talk to
	 * @throws IOException if the output stream could not be created
	 * @throws ConfigurationException if the configured numeric is not a valid numeric
	 */
	public DarenetProtocol(final Socket sock) throws IOException, ConfigurationException
	{
		super(sock);

		long in;
		String numeric;

		try {
			in = Long.parseLong(TheXBot.conf.getSID());
		} catch (NumberFormatException e) {
			throw new ConfigurationException(TheXBot.conf.getSID() + " is not a valid number.");
		}
		numeric = Util.inttobase64((int)in, 2);
		this.me = new Server(TheXBot.conf.getServerName(), TheXBot.conf.getServerDescription(),
			numeric, null);

		this.message_handlers = new HashMap<>();
		this.message_handlers.put("SERVER", new SERVERHandler(this));
		this.message_handlers.put("N", new NHandler(this));
		this.message_handlers.put("B", new BHandler(this));
		this.message_handlers.put("S", new SHandler(this));
		this.message_handlers.put("EB", new EBHandler(this));
		this.message_handlers.put("J", new JHandler(this));
		this.message_handlers.put("C", new CHandler(this));
		this.message_handlers.put("G", new GHandler(this));
		this.message_handlers.put("SQ", new SQHandler(this));
		this.message_handlers.put("ERROR", new ERRORHandler(this));
		this.message_handlers.put("L", new LHandler(this));
		this.message_handlers.put("K", new KHandler(this));
		this.message_handlers.put("Q", new QHandler(this));
		this.message_handlers.put("P", new PHandler(this));
		this.message_handlers.put("W", new WHandler(this));
		this.message_handlers.put("AC", new ACHandler(this));
		this.message_handlers.put("SN", new SNHandler(this));
		this.message_handlers.put("SJ", new SJHandler(this));
		this.message_handlers.put("MK", new MKHandler(this));

		/* Not implementing D because we have no pseudoclients to revive and they should be
		 * protected by +k anyway.
		 */

		MessageHandler mode_handler = new MHandler(this);
		this.message_handlers.put("M", mode_handler);
		this.message_handlers.put("OM", mode_handler);
	}

	@Override
	public Message parse(String line) throws ParserException
	{
		if (line == null) {
			return null;
		}

		if (line.indexOf(':') == 0) {
			return super.parse(line);
		}

		char[] buf = line.toCharArray();
		int colonat = line.indexOf(" :") + 1;
		if (colonat == 0) { // Not found, but added +1
			colonat = -1;
		}
		int spaces = 0;

		if (colonat != -1) {
			for (int i = 0; i < colonat; i++) {
				if (buf[i] == ' ') {
					spaces++;
				}
			}
		} else {
			// Don't split after the colon. To do so, keep track of the spaces.
			for (int i = 0; i < buf.length; i++) {
				if (buf[i] == ' ') {
					spaces++;
				}
			}
		}

		// ircd-darenet seems to send double spaces at times
		String[] pieces = line.split("\\s+", spaces + 1);
		boolean has_source = (!pieces[0].equals("PASS") && !pieces[0].equals("SERVER"));
		if (pieces[pieces.length - 1].startsWith(":")) {
			pieces[pieces.length - 1] = pieces[pieces.length - 1].substring(1);
		}

		String source = has_source ? pieces[0] : null;
		String cmd = has_source ? pieces[1] : pieces[0];
		String[] args = Arrays.copyOfRange(pieces, has_source ? 2 : 1, pieces.length);

		log.log(Level.FINEST,
			"src: " + (source == null ? "(null)" : source) + " cmd: " + cmd
			+ " arg1: " + (args.length == 0 ? "(empty)" : args[0]));

		return new Message(source, cmd.toUpperCase(), args);
	}

	@Override
	public void parseUserModes(final User u, String[] modearray)
	{
		if (modearray == null || modearray.length == 0) {
			return;
		}
		int offset = 1;
		boolean add = modearray[0].charAt(0) == '+';

		for (final char c : modearray[0].toCharArray()) {
			switch (c) {
				case '+':
					add = true;
					break;
				case '-':
					add = false;
					break;
				case 'd':
					if (add) {
						u.setMode(UserMode.DEAF);
					} else {
						u.unsetMode(UserMode.DEAF);
					}
					break;
				case 'k':
					if (add) {
						u.setMode(UserMode.SERVICE);
					} else {
						// theoretically impossible, but eh
						u.unsetMode(UserMode.SERVICE);
					}
					break;
				case 'N':
					if (add) {
						u.setMode(UserMode.NETADMIN);
					} else {
						u.unsetMode(UserMode.NETADMIN);
					}
					break;
				case 'a':
					if (add) {
						u.setMode(UserMode.ADMIN);
					} else {
						u.unsetMode(UserMode.ADMIN);
					}
					break;
				case 'o':
					// in darenet2, this would carry the oper name
					if (add) {
						u.setMode(UserMode.OPER);
					} else {
						u.unsetMode(UserMode.OPER);
					}
					break;
				//case 'f': darenet2
				case 'h':
					if (add) {
						u.setShownHost(modearray[offset]);
						u.setMode(UserMode.VHOST);
					} else {
						throw new IllegalStateException("Removal of +h should be impossible");
					}
					offset++;
					break;
				case 'r':
					if (add) {
						String[] parts = modearray[offset].split(":");
						if (parts.length == 0) {
							log.log(Level.INFO, "Received +r without arg, ignoring.");
							break;
						}
						// darenet1: name:creationts:id, but all args optional
						AccountInfo ai;
						if (parts.length > 2) {
							ai = new AccountInfo(parts[0],
								Integer.parseInt(parts[2]),
								Integer.parseInt(parts[1]));
						} else if (parts.length == 2) {
							ai = new AccountInfo(parts[0], -1,
								Integer.parseInt(parts[1]));
						} else if (parts.length == 1) {
							ai = new AccountInfo(parts[0], -1, -1);
						} else {
							throw new IllegalStateException("Final else in parseUserMode +r");
						}
						u.setMetaData("account", ai);
					} else {
						u.removeMetaData("account");
					}
					offset++;
					break;
				case 'x':
					if (add) {
						if (!u.isAuthed()) {
							break;
						}
						u.setMode(UserMode.HIDEHOST);
						hideUserHost(u);
					} else {
						throw new IllegalStateException("Removal of +x should be impossible");
					}
					break;
				case 'i':
					if (add) {
						u.setMode(UserMode.INVISIBLE);
					} else {
						u.unsetMode(UserMode.INVISIBLE);
					}
					break;
				default:
					break;
			}
		}
	}

	/**
	 * Returns the String representation of a mode set. Note that this is <em>not</em> the order
	 * that the IRCd uses and it may very likely mess things up if sent back to the IRCd.
	 *
	 * @param u the user whose modes are to be stringified
	 * @return a mode string
	 */
	@Override
	public String getUserModeString(User u)
	{
		StringBuilder sb = new StringBuilder("");
		for (final UserMode m : u.getModes()) {
			switch (m) {
				case ADMIN:
					sb.append('a');
					break;
				case DEAF:
					sb.append('d');
					break;
				case HIDEHOST:
					sb.append('x');
					break;
				case INVISIBLE:
					sb.append('i');
					break;
				case NETADMIN:
					sb.append('N');
					break;
				case OPER:
					sb.append('o');
					break;
				case SERVICE:
					sb.append('k');
					break;
				case VHOST:
					sb.append('h');
					break;
				default:
					break;
			}
		}

		AccountInfo ai = u.getAccountInfo();
		if (ai != null) {
			sb.append("r ");
			sb.append(ai.getName());
			if (ai.getCreated() != -1) {
				sb.append(':');
				sb.append(ai.getCreated());
				if (ai.getID() != -1) {
					sb.append(':');
					sb.append(ai.getID());
				}
			}
		}

		return sb.toString();
	}

	@Override
	public void parseChannelModes(final Channel c, String[] modearray)
	{
		if (modearray == null || modearray.length == 0) {
			return;
		}
		//int offset = 1;
		boolean add = modearray[0].charAt(0) == '+';

		for (final char m : modearray[0].toCharArray()) {
			switch (m) {
				case '+':
					add = true;
					break;
				case '-':
					add = false;
					break;
				case 'z':
					c.setPersistent(add);
					break;
				default:
					break;
			}
		}
	}

	@Override
	public void hideUserHost(final User u)
	{
		if (!u.isAuthed() || u.hasMode(UserMode.VHOST) || !u.hasMode(UserMode.HIDEHOST)) {
			return;
		}

		if (TheXBot.conf.getUserCloak() == null || TheXBot.conf.getOperCloak() == null) {
			log.log(Level.WARNING, "User or oper cloak configuration settings not set! Expect host desyncs on our part");
			return;
		}

		// darenet2: separate user/oper cloaks
		u.setShownHost(u.getAccountInfo().getName() + "."
			+ (u.hasMode(UserMode.OPER) ? TheXBot.conf.getOperCloak() : TheXBot.conf.getUserCloak()));
	}

	@Override
	public void interpret(final Message message)
	{
		if (message == null) {
			log.log(Level.FINER, "Got null message.");
			return;
		}

		MessageHandler mh = message_handlers.get(message.command);

		if (mh == null) {
			log.log(Level.FINE, "Received unknown message: " + message.command);
			return;
		}

		if (mh.getMinimumArgumentCount() > message.args.length) {
			log.log(Level.WARNING, "Received " + message.args.length + " params to "
				+ message.command + " (expected " + mh.getMinimumArgumentCount()
				+ "). Desyncs possible.");
			return;
		}

		Server server = null;
		User user;
		boolean any = false;
		try {
			if (mh.getSourceType() != SourceType.NONE && message.source == null) {
				log.log(Level.FINE, "Got message " + message + " with source? Dropping.");
				return;
			}
			switch (mh.getSourceType()) {
				case NONE:
					mh.handle(message);
					break;
				case ANY:
					any = true;
				// fallthrough
				case SERVER:
					server = TheXBot.em().findServer(message.source);

					if (!any && server == null) {
						// Level.INFO because this can legitimately happen pre-SQUIT
						log.log(Level.INFO, "Got message for non-existent server "
							+ message.source);
						break;
					}
					mh.handle(server, message);
					if (!any) {
						break;
					}
				// fallthrough
				case USER:
					user = TheXBot.em().findUser(message.source);

					if (user != null && server != null) {
						log.log(Level.WARNING,
							"Found user *and* server, dropping message from: "
							+ message.source);
						break;
					}

					if (any && server != null) {
						mh.handle(server, message);
						break;
					}

					if (user == null) {
						// Level.INFO because this can legitimately happen pre-SQUIT
						log.log(Level.INFO, "Got message for non-existent user "
							+ message.source);
						break;
					}

					mh.handle(user, message);
					break;
			}
		} catch (Exception e) {
			log.log(Level.SEVERE, "Exception while handling message " + message, e);
		}
	}

	@Override
	public Server getServer()
	{
		return this.me;
	}
}
