package invalid.thexbot.protocol;

import invalid.thexbot.Util;

/**
 * Defines a raw message as received from the IRC server.
 *
 * @since 0.5.0
 */
public final class Message {
	/**
	 * Source of the message. May be null for certain messages.
	 *
	 * @since 0.5.0
	 */
	public final String source;
	/**
	 * The main command that was received. Always upper-case here.
	 *
	 * @since 0.5.0
	 */
	public final String command;
	/**
	 * The list of arguments to the command. The final argument will have its colon stripped
	 *
	 * @since 0.5.0
	 */
	public final String[] args;

	/**
	 * Initializes a new message.
	 *
	 * @param source source of the command (may be null)
	 * @param command the actual command (not null)
	 * @param args the arguments to the command (not null, but may be empty)
	 * @since 0.5.0
	 */
	public Message(final String source, final String command, final String[] args)
	{
		this.source = source;
		this.command = command.toUpperCase();
		this.args = args;
	}
	
	@Override
	public String toString()
	{
		return this.command + " (from " + this.source + "): " + Util.join(" ", args);
	}
}
