package invalid.thexbot.protocol.darenetprotocol;

import java.util.EnumSet;
import java.util.logging.Level;
import invalid.thexbot.Logger;
import invalid.thexbot.TheXBot;
import invalid.thexbot.irc.Channel;
import invalid.thexbot.irc.ChannelUserStatus;
import invalid.thexbot.irc.Server;
import invalid.thexbot.irc.User;
import invalid.thexbot.protocol.Message;
import invalid.thexbot.protocol.MessageHandler;
import invalid.thexbot.protocol.Protocol;

public final class BHandler extends MessageHandler {
	private static final Logger log = Logger.getLogger(BHandler.class.getName());

	public BHandler(Protocol p)
	{
		super(p, 2, SourceType.SERVER);
	}

	@Override
	public void handle(User u, Message m)
	{
		throw new UnsupportedOperationException("Handler called for unsupported source.");
	}

	@Override
	public void handle(Server s, Message m)
	{
		// AZ B #darenet 949217470 +tinkl key 56 AAAAA,AAAAB,AAAAC,ABAAA,ABAAB,ABAAC,ACAAA :%*!*@*.net ~ *!*@*.com
		//      0        1         2      3   4  5                                         6

		String channel = m.args[0];
		int ts = Integer.parseInt(m.args[1]);

		Channel chan = TheXBot.em().findChannel(channel);
		if (chan == null) {
			chan = new Channel(channel, 0);
		}

		// Warning: Everything after the first arg is completely unreliable

		if (chan.getTS() == 0) {
			if (m.args.length == 2) { // empty channel without +A
				if (ts < chan.getTS() && chan.getTS() <= (ts + 4) && !chan.getUsers().isEmpty()) {
					ts = chan.getTS();
				}
			} else if (chan.getTS() < ts && ts <= (ts + 4) && !chan.getUsers().isEmpty()) {
				chan.setTS(ts);
			}
		}

		if (chan.getTS() == 0 || chan.getTS() > ts) {
			chan.setTS(ts);
			// We don't care about simple channel modes, skip those

			// uplink kicks any potential split riders for us, we have nothing to do for mode handling

			// We don't actually need to honor the TS for our local clients getting possibly deopped
		}

		int offset = 2;
		for (int i = 2; i < m.args.length; i++) {
			if (m.args[i].charAt(0) == '+') /* modes */ {
				offset = i;
				for (char c : m.args[i].toCharArray()) {
					switch (c) {
						case 'k':
						case 'l':
							offset++;
						default:
							break;
					}
				}
				continue;
			}

			if (m.args[i].charAt(0) == '%' /* bans */) {
				continue;
			}

			// *SHOULD* be client list now, but make sure we're below offset
			if (i <= offset) {
				continue;
			}

			String[] uparams = m.args[i].split(",");
			User[] users = new User[uparams.length];
			EnumSet<ChannelUserStatus> last_status = EnumSet.noneOf(ChannelUserStatus.class);
			
			// Must be client list now
			for (int j = 0; j < uparams.length; j++) {
				String user = uparams[j];
				// A8AAA:ov123 -- but disregard oplevels!
				int idx = user.indexOf(':');
				String uid = user.substring(0, (idx == -1) ? user.length() : idx);

				User u = TheXBot.em().findUser(uid);
				if (u == null) {
					log.log(Level.WARNING, "Unknown UID " + uid);
					continue;
				}

				EnumSet<ChannelUserStatus> modes = EnumSet.noneOf(ChannelUserStatus.class);

				if (idx != -1) { // has CUS info
					for (char c : user.substring(idx + 1, user.length()).toCharArray()) {
						switch (c) {
							case 'o':
							case '0':
							case '1':
							case '2':
							case '3':
							case '4':
							case '5':
							case '6':
							case '7':
							case '8':
							case '9':
								modes.add(ChannelUserStatus.CHANOP);
								break;
							case 'h':
								modes.add(ChannelUserStatus.HALFOP);
								break;
							case 'v':
								modes.add(ChannelUserStatus.VOICE);
								break;
							default:
								log.log(Level.WARNING, "Unknown mode in " + user.substring(idx + 1, user.length()));
								break;
						}
					}
					last_status = modes;
				} else { // no CUS info: same as last user (or peon)
					modes = last_status;
				}

				chan.addUser(u, modes);
				u.join(chan);
				users[j] = u;
			}

			TheXBot.em().addChannel(chan);
		}

	}

	@Override
	public void handle(Message m)
	{
		throw new UnsupportedOperationException("Handler called for unsupported source.");
	}
}
