package invalid.thexbot.protocol.darenetprotocol;

import java.util.Arrays;
import java.util.logging.Level;
import invalid.thexbot.Logger;
import invalid.thexbot.TheXBot;
import invalid.thexbot.irc.Channel;
import invalid.thexbot.irc.Server;
import invalid.thexbot.irc.User;
import invalid.thexbot.protocol.Message;
import invalid.thexbot.protocol.MessageHandler;
import invalid.thexbot.protocol.Protocol;

public final class MHandler extends MessageHandler {
	private static final Logger log = Logger.getLogger(MHandler.class.getName());
	public MHandler(Protocol p)
	{
		super(p, 1, SourceType.ANY);
	}

	@Override
	public void handle(User u, Message m)
	{
		if (m.args[0].charAt(0) != '#') {
			User target = TheXBot.em().findUser(m.args[0]);
			if (target == null) {
				log.log(Level.WARNING,
					"User mode target " + m.args[0] + " not found.");
				return;
			}

			this.protocol.parseUserModes(target,
				Arrays.copyOfRange(m.args, 0, m.args.length));
			return;
		}

		Channel target = TheXBot.em().findChannel(m.args[0]);
		if (target == null) {
			log.log(Level.WARNING,
				"Channel mode target " + m.args[0] + " not found.");
			return;
		}
		
		try {
			int ts = Integer.parseInt(m.args[m.args.length - 1]);
			if (ts != 0 && ts > target.getTS()) {
				log.log(Level.WARNING, "Dropping mode change for "
					+ target.getName() + ", newer TS");
			}
		} catch (NumberFormatException e) {
			// Nothing, we don't care
		}
		
		this.protocol.parseChannelModes(target, Arrays.copyOfRange(m.args, 2, m.args.length - 1));
	}

	@Override
	public void handle(Server s, Message m)
	{
		throw new UnsupportedOperationException("Handler called for unsupported source.");
	}

	@Override
	public void handle(Message m)
	{
		throw new UnsupportedOperationException("Handler called for unsupported source.");
	}
}
