package invalid.thexbot.protocol.darenetprotocol;

import java.util.Arrays;
import invalid.thexbot.TheXBot;
import invalid.thexbot.irc.PseudoClient;
import invalid.thexbot.irc.Server;
import invalid.thexbot.irc.User;
import invalid.thexbot.irc.events.Command;
import invalid.thexbot.protocol.Message;
import invalid.thexbot.protocol.MessageHandler;
import invalid.thexbot.protocol.Protocol;

public final class PHandler extends MessageHandler {
	public PHandler(Protocol p)
	{
		super(p, 2, SourceType.ANY);
	}

	@Override
	public void handle(User u, Message m)
	{
		PseudoClient client = TheXBot.em().findPseudoClient(m.args[0]);
		if (client == null) {
			return;
		}

		String[] parts = m.args[1].split("\\s+");
		if (parts.length == 0) {
			return;
		}

		String subcmd = parts[0].toUpperCase();
		
		if (subcmd.equals("HELP")) {
			if (parts.length < 2) {
				client.notice(u, "List of commands:");
				for (final Command c : client.getCommands()) {
					client.notice(u, String.format("  \002%-13s\002 / %s", 
						c.getName(), c.getShortHelp()));
				}
				client.notice(u, "End of command list.");
				return;
			} else {
				Command cmd = client.findCommand(parts[1]);
				if (cmd == null) {
					client.notice(u, "No help available for \002" + parts[1] + "\002.");
					return;
				}
				
				cmd.fullHelp(u);
			}
			return;
		}
		
		Command cmd = client.findCommand(subcmd);
		if (cmd == null) {
			this.protocol.sendNotice(client, u, "Unknown command \002" + subcmd + "\002.");
			return;
		}

		cmd.run(u, (parts.length == 1 ? new String[0] : Arrays.copyOfRange(parts, 1, parts.length)));
	}

	@Override
	public void handle(Server s, Message m)
	{
		return;
	}

	@Override
	public void handle(Message m)
	{
		throw new UnsupportedOperationException("Handler called for unsupported source.");
	}
}
