package invalid.thexbot.scripting;

import java.util.logging.Level;
import invalid.thexbot.Logger;
import invalid.thexbot.TheXBot;
import invalid.thexbot.exceptions.ModuleException;
import invalid.thexbot.exceptions.ResourceException;
import invalid.thexbot.irc.PseudoClient;
import invalid.thexbot.irc.events.EventHandler;
import invalid.thexbot.protocol.Protocol;

/**
 * Base class all scripted modules inherit from.
 *
 * @since 0.5.0
 */
public abstract class Module extends EventHandler {
	/**
	 * Creates a new {@link PseudoClient} for this module. This method will add the client to
	 * the user/server lists and all that.
	 *
	 * @param protocol the {@link Protocol} instance we use
	 * @param nick the nick for the new client
	 * @param ident the ident for the new client
	 * @param host the host for the new client
	 * @param gecos the GECOS (also known as ircname/realname) for the new client
	 * @return a new PseudoClient
	 * @throws ResourceException if we ran out of user numerics
	 * @since 0.5.0
	 */
	protected PseudoClient createPseudoClient(Protocol protocol, String nick, String ident,
		String host, String gecos) throws ResourceException
	{
		PseudoClient u = new PseudoClient(protocol, nick, ident, host, gecos);

		TheXBot.em().addPseudoClient(u);

		return u;
	}

	/**
	 * Removes a {@link PseudoClient} from X.
	 *
	 * @param client the client to remove
	 */
	protected void removePseudoClient(final PseudoClient client)
	{
		TheXBot.em().removePseudoClient(client);
	}

	/**
	 * Initializes the module as necessary. You should not call this manually, as it will be
	 * done by the module itself.
	 *
	 * @param protocol the {@link Protocol} instance the module will use
	 */
	public abstract boolean __init__(Protocol protocol) throws ModuleException;

	/**
	 * Causes the module to clean up.
	 */
	public abstract void __deinit__();
}
